<html>
    <head>
        <title>Monzo API</title>
    </head>
    <body>
        <h1>Monzo API</h1>
        <p>Monzo is now set up. Click <a href="/accounts/">HERE</a> to get started</p>
    </body>
</html>
