<html>
    <head>
        <title>Monzo API</title>
    </head>
    <body>
        <h1>Monzo API</h1>
        <h2>Transactions</h2>
        {% if transactions %}
        <table>
            <tr>
                <th>Merchant</th>
                <th>Description</th>
                <th>Paid On</th>
                <th>Amount</th>
            </tr>
            {% for transaction in transactions%}
            <tr>
                <td>{{ transaction.merchant.name }}</td>
                <td>{{ transaction.description }}</td>
                <td>{{ transaction.created }}</td>
                <td>{{ transaction.currency }}{{ "%.2f"|format(transaction.amount / 100) }}</td>
            </tr>
            {% endfor %}
        </table>
        {% endif %}
    </body>
</html>
