<html>
    <head>
        <title>Monzo API</title>
    </head>
    <body>
        <h1>Monzo API</h1>
        <p>Prior to being able to query the API you first need to set up an Access token. To find out how to do this you
            can follow this <a href="https://monzo-api.readthedocs.io/en/latest/monzo_setup.html#">handy tutorial</a>.</p>
        <p>When setting up be sure to set the redirect URL to be <b>{{ REDIRECT_URL }}</b></p>
        <p id="error">{{ ERROR_MESSAGE }}</p>
        <form method="post" action="/setup/">
            <label for="client_id">Client ID</label>
            <input type="text" value="{{ CLIENT_ID_VALUE }}" id="client_id" name="client_id" size="50" /><br />

            <label for="client_secret">Client Secret</label>
            <input type="text" value="{{ CLIENT_SECRET_VALUE }}" id="client_secret" name="client_secret" size="50" /><br />

            <input type="submit" name="Submit">
        </form>
    </body>
</html>
