<html>
    <head>
        <title>Monzo API</title>
    </head>
    <body>
        <h1>Monzo API</h1>
        <h2>Raw Request</h2>
        <form method="post">
            <label for="path">Path</label>
            <input type="text" name="path" id="path" /><br />
            <label for="request_type">Request Type</label>
            <select name="request_type" id="request_type">
                <option value="delete">Delete</option>
                <option value="get" selected>Get</option>
                <option value="post">Post</option>
                <option value="put">Put</option>
            </select><br />
            <label for="authenticated">Authenticated</label>
            <select name="authenticated" id="authenticated">
                <option value="no">No</option>
                <option value="yes" selected>Yes</option>
            </select><br />
            <label for="parameters">Parameters</label>
            <textarea name="parameters" id="parameters"></textarea><br />
            <button type="submit" name="submit" value="submit">Submit</button>
        </form>
            {% if records %}
                <pre>
                    {{ records }}
                </pre>
            {% endif %}
    </body>
</html>
